C**********************************************************************
C**********************************************************************
C** 
C**     File: tiff.for
C**
C**     Project: jitax77
C**
C**     Description: Routines for a minimal TIFF reader
C**
C**  TIFF documentation is available from:
C**     http://partners.adobe.com/asn/developer/pdfs/tn/TIFF6.pdf
C**
C**********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C** This file is part of jitax77.
C**
C** jitax77 is free software; you can redistribute it and/or modify
C** it under the terms of the GNU General Public License as published
C** by the Free Software Foundation; either version 2 of the License,
C** or (at your option) any later version.
C**
C** jitax77 is distributed in the hope that it will be useful,
C** but WITHOUT ANY WARRANTY; without even the implied warranty of
C** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C** GNU General Public License for more details.
C**
C** You should have received a copy of the GNU General Public License
C** along with jitax77; if not, write to the Free Software
C** Foundation, Inc., 59 Temple Place, Suite 330, Boston,
C** MA  02111-1307  USA.
C**
C**********************************************************************
C**********************************************************************
C
C**********************************************************************
C     Routine: OPEN_TIFF_FILE
C
C     Description:
C        Opens the tiff file whose name is stored in NAME
C        NB. Uses a fixed unit number. This means only one TIFF
C        can be open at a time. CLOSE_TIFF_FILE must be called
C        before OPEN_TIFF_FILE can be called again.
C
C        Reads in header information and stores it in the COMMONS
C        found in tiff.inc
C        If DEBUG is .TRUE. then extra debug information will be
C        printed to the terminal
C
C        Returns: 0 file successfully opened, 
C                 1 error opening file
C                 2 error reading TIFF header
C                 3 error parsing TIFF header
C                 4 error reading IFD
C                 5 error reading TIFF data
C**********************************************************************
      INTEGER FUNCTION OPEN_TIFF_FILE(NAME, DEBUG)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C
      CHARACTER*(*) NAME        ! TIFF file name
      LOGICAL       DEBUG       ! Debug mode
C
C     Functions
C
      INTEGER READ_TIFF_FILE
      INTEGER READ_TIFF_SHORT
      INTEGER READ_TIFF_LONG
      INTEGER READ_TIFF_IFD
      INTEGER READ_TIFF_DATA
      LOGICAL BIGEND_CPU      
C
C     Variables
C
      LOGICAL   OPEN            ! .TRUE. if a file is already open
      BYTE      HEADER(8)       ! Store the 8-byte TIFF header
      INTEGER   MAGIC           ! TIFF magic num. Should be 42
      INTEGER   IFD1_OFFSET     ! The byte offset of the first IFD
C
C     Initialise the LAST_REC common. 0 means that BUFFER contains
C     no valid record.
C
      LAST_REC = 0
C
C     Check that a file isn't currently open. These routines
C     can only work with one file at a time.
C
      INQUIRE(UNIT=UNT, OPENED=OPEN)
      IF (OPEN) THEN
         WRITE(*,*) '** OPEN_TIFF_FILE error: File already opened.'
         WRITE(*,*) '** Only one TIFF file can be opened at a time.'
         GOTO 901
      ENDIF
C
C     Open file
C
      OPEN(UNIT=UNT, FILE=NAME, STATUS='OLD', FORM='UNFORMATTED',
     $     ACCESS='DIRECT', RECL = REC_SIZE, ERR=901)
      REWIND(UNIT=UNT)
C
C     Read TIFF header
C
      IF (READ_TIFF_FILE(HEADER, 0, 7) .NE. 0) GOTO 902
C
C     The first two bytes of a TIFF header should be II or MM
C     = 73, 73 or = 77, 77
C
      IF (HEADER(1) .EQ. 73 .AND. HEADER(2) .EQ. 73) THEN
         BIG_END = .FALSE.
      ELSE IF (HEADER(1) .EQ. 77 .AND. HEADER(2) .EQ. 77) THEN
         BIG_END = .TRUE.
      ELSE
         GOTO 903
      ENDIF
C
C     If the machine has a different endian than the file
C     then set SWAP to .TRUE.
C
      SWAP = (BIGEND_CPU() .XOR. BIG_END)
      IF (DEBUG) WRITE(*,*) 'BIG_END = ', BIG_END, ' SWAP = ', SWAP
C
C     Check magic number
C
      MAGIC = READ_TIFF_SHORT(HEADER(3))
      IF (DEBUG) WRITE(*,*) 'MAGIC = ', MAGIC
      IF (MAGIC .NE. 42) GOTO 903
C
C     Read the first IFD.
C
      IFD1_OFFSET = READ_TIFF_LONG(HEADER(5))
      IF (DEBUG) WRITE(*,*) 'IFD1_OFFSET = ', IFD1_OFFSET
      IF (READ_TIFF_IFD(IFD1_OFFSET, DEBUG) .NE. 0) GOTO 904
C
      OPEN_TIFF_FILE = 0
      RETURN
C
C     Error traps
C
 901  OPEN_TIFF_FILE = 1        ! Error opening file
      RETURN
 902  OPEN_TIFF_FILE = 2        ! Error reading header
      RETURN
 903  OPEN_TIFF_FILE = 3        ! Error parsing TIFF header
      RETURN
 904  OPEN_TIFF_FILE = 4        ! Error reading IFD
      RETURN
 905  OPEN_TIFF_FILE = 5        ! Error reading TIFF data
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_FILE
C
C     Description:
C        Reads bytes START_BYTE to END_BYTE, inclusive, from a TIFF file 
C        into DATA.
C        NB. START_BYTE and END_BYTE are zero offset.
C     
C        Returns 0 if successful, 1 otherwise
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_FILE(DATA, START_BYTE, END_BYTE)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C     
      BYTE    DATA(*)           ! The array to read the bytes into
      INTEGER START_BYTE        ! The byte to start reading from
      INTEGER END_BYTE          ! The byte to finish reading at.
C
C     Functions
C
      INTEGER READ_TIFF_RECORD
C
C     Variables
C          
      INTEGER START_REC         ! Start record
      INTEGER END_REC           ! End record
      INTEGER TOTAL_BYTES       ! The total number of bytes to read
      INTEGER NEXT_BYTE         ! The next byte to write into DATA
      INTEGER CUR_REC           ! Current record being read
      INTEGER SKIP_BYTES        ! Num bytes to skip in first record
      INTEGER NUM_BYTES         ! Num bytes to read from first record
      INTEGER STATUS            ! Status of the read
      INTEGER I                 ! Loop counter
C     
C     Check for errors in the parameters START_BYTE and END_BYTE.
C     
      IF ((START_BYTE .GT. END_BYTE) .OR. (START_BYTE .LT. 0)) THEN
         WRITE(*,*) '** READ_TIFF_FILE Error: Bad START_BYTE or
     $        END_BYTE'
         WRITE(*,*) '** START = ', START_BYTE, ', END = ', END_BYTE
         GOTO 911
      ENDIF
C     
C     Calculate the first and last records that have to be read, and
C     the total number of bytes that have to be read.
C     
      START_REC   = 1 + START_BYTE/REC_SIZE
      END_REC     = 1 + END_BYTE/REC_SIZE
      TOTAL_BYTES = END_BYTE - START_BYTE + 1
C     
      NEXT_BYTE = 1
      CUR_REC   = START_REC
C     
C     Read the first record (unless it already is in BUFFER).  
C     
      STATUS = 0
      IF(CUR_REC .NE. LAST_REC) THEN
         READ(UNIT=UNT, REC=CUR_REC, IOSTAT=STATUS) BUFFER
      ENDIF
C     
C     Copy the required bytes from BUFFER to DATA.
C     
      SKIP_BYTES = START_BYTE - (START_REC - 1)*REC_SIZE
      NUM_BYTES  = MIN(REC_SIZE - SKIP_BYTES, TOTAL_BYTES)
      DO I = 1, NUM_BYTES
         DATA(I) = BUFFER(I + SKIP_BYTES)
      ENDDO
      NEXT_BYTE = NEXT_BYTE + NUM_BYTES
      IF(STATUS .NE. 0) GOTO 911
C     
C     Read anymore whole records directly into DATA.
C     
      DO CUR_REC = START_REC + 1, END_REC - 1
         STATUS = READ_TIFF_RECORD(DATA(NEXT_BYTE), CUR_REC)
         IF(STATUS .NE. 0) GOTO 911
         NEXT_BYTE = NEXT_BYTE + REC_SIZE
      ENDDO
C     
C     If needed, read the last record into BUFFER, then transfer to DATA.
C     
      CUR_REC = END_REC
      IF(CUR_REC .NE. START_REC) THEN
         READ(UNIT=UNT, REC=CUR_REC, IOSTAT=STATUS) BUFFER
         DO I = NEXT_BYTE, TOTAL_BYTES
            DATA(I) = BUFFER(I - NEXT_BYTE + 1)
         ENDDO
      ENDIF
C     
C     Store the record number that BUFFER contains, then return.
C     
      LAST_REC = CUR_REC
      READ_TIFF_FILE = 0
      RETURN
C     
C     Error occured. Set LAST_REC = 0 so we don't reuse BUFFER contents.
C
C     Ignore errors that occur when reading END_REC since this
C     might be the last record in the file and may cause an error
C     because it is less than REC_SIZE in size.
C     
 911  IF (CUR_REC .EQ. END_REC) THEN
         LAST_REC = CUR_REC
         READ_TIFF_FILE =0
      ELSE
         LAST_REC = 0
         READ_TIFF_FILE = 1
      ENDIF
      RETURN
C     
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_RECORD
C
C     Description:
C        Reads CUR_REC for the currently open TIFF file into
C        ARRAY.
C     
C        Returns 0 if successful, 1 otherwise
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_RECORD(ARRAY, RECORD)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameter
C
      BYTE    ARRAY(REC_SIZE)
      INTEGER RECORD
C
C     Variables
C
      INTEGER STATUS
C
      READ(UNIT=UNT, REC=RECORD, IOSTAT=STATUS) ARRAY
      READ_TIFF_RECORD=STATUS
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_IFD
C
C     Description:
C        Reads the TIFF IFD starting at byte IFD_OFFSET
C        NB. IFD_OFFSET starts at byte 0
C     
C        Returns 0 if successful, 1 otherwise
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_IFD(IFD_OFFSET, DEBUG)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C     
      INTEGER IFD_OFFSET        ! The byte offset of the IFD
      LOGICAL DEBUG             ! Debug mode
C
C     Functions
C
      INTEGER READ_TIFF_FILE
      INTEGER READ_TIFF_SHORT
      INTEGER PARSE_IFD_ENT
C
C     Variables
C
      BYTE     BUF2(2)          ! 2-byte buffer
      BYTE     ENT(12)          ! Directory entry buffer
      INTEGER  NUMENT           ! Number of directory entries
      INTEGER  I                ! Loop counter
C
C     Read number of directory entries in IFD into raw buffer
C
      IF (READ_TIFF_FILE(BUF2, IFD_OFFSET, IFD_OFFSET+1) .NE. 0) THEN
         GOTO 921
      ENDIF
C
C     Convert raw buffer into number
C     
      NUMENT = READ_TIFF_SHORT(BUF2)
      IF (DEBUG) WRITE(*,*) 'NUMENT = ', NUMENT
C
      DO I = 1, NUMENT
         IF (READ_TIFF_FILE(ENT, 
     $                      IFD_OFFSET+2+(I-1)*12,
     $                      IFD_OFFSET+2+I*12-1) .NE. 0) GOTO 921
         IF (PARSE_IFD_ENT(ENT, DEBUG) .NE. 0) GOTO 921
      ENDDO
C
      READ_TIFF_IFD = 0
      RETURN
C
C     Error trap
C
 921  READ_TIFF_IFD = 1
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: PARSE_IFD_ENT
C
C     Description:
C        Parses a TIFF IFD Entry and saves the tags that we are
C        interested in into the COMMON in tiff.inc
C
C        Returns 0 if successful, 1 otherwise.
C
C        NB. This function can quit on fatal errors (i.e. TIFF file
C            types this reader can't handle.
C**********************************************************************
      INTEGER FUNCTION PARSE_IFD_ENT(ENT, DEBUG)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C
      BYTE    ENT(12)           ! The 12-bytes IFD entry to parse
      LOGICAL DEBUG             ! Debug mode
C
C     Functions
C
      INTEGER READ_TIFF_FILE
      INTEGER READ_TIFF_SHORT
      INTEGER READ_TIFF_LONG
      INTEGER STRLEN
C
C     Variables
C
      INTEGER TAG               ! TIFF tag
      INTEGER TYPE              ! Tag type: 1 = BYTE, 2 = ASCII
                                !           3 = SHORT, 4 = LONG
                                !           5 = RATIONAL
      INTEGER COUNT             ! Number of values
      INTEGER IVAL              ! Integer value
      INTEGER OFFSET            ! Location of multi-value tag data
      INTEGER I                 ! Loop counter
      BYTE    BUF1              ! 1-byte buffer
      BYTE    BUF2(2)           ! 2-byte buffer
      BYTE    BUF4(4)           ! 4-byte buffer
C
      TAG   = READ_TIFF_SHORT(ENT(1))
      TYPE  = READ_TIFF_SHORT(ENT(3))
      COUNT = READ_TIFF_LONG(ENT(5))
C
C     Parse known tags.
C
      IF (TAG .EQ. 254) THEN
         IVAL = READ_TIFF_LONG(ENT(9))
         NEWSUBFILETYPE = IVAL
         IF (DEBUG) WRITE(*,*) 'NewSubfileType = ', IVAL
      ELSE IF (TAG .EQ. 256) THEN
         IF (TYPE .EQ. 3) IVAL = READ_TIFF_SHORT(ENT(9))
         IF (TYPE .EQ. 4) IVAL = READ_TIFF_LONG(ENT(9))
         WIDTH = IVAL
         IF (DEBUG) WRITE(*,*) 'ImageWidth = ', IVAL
      ELSE IF (TAG .EQ. 257) THEN
         IF (TYPE .EQ. 3) IVAL = READ_TIFF_SHORT(ENT(9))
         IF (TYPE .EQ. 4) IVAL = READ_TIFF_LONG(ENT(9))
         HEIGHT = IVAL
         IF (DEBUG) WRITE(*,*) 'ImageHeight = ', IVAL
      ELSE IF (TAG .EQ. 258) THEN
         IF (COUNT .EQ. 1) THEN
            IVAL = READ_TIFF_SHORT(ENT(9))
            BITSPERSAMPLE = IVAL
            IF (DEBUG) WRITE(*,*) 'BitsPerSample = ', IVAL
         ELSE
            WRITE(*,*) '** PARSE_IFD_ENT: Unsupported TIFF file type'
            WRITE(*,*) '** Cannot handle more than one sample per pixel'
            WRITE(*,*) '** Quitting'
            STOP
         ENDIF
      ELSE IF (TAG .EQ. 259) THEN
         IVAL = READ_TIFF_SHORT(ENT(9))
         COMPRESSION = IVAL
         IF (DEBUG) WRITE(*,*) 'Compression = ', IVAL
      ELSE IF (TAG .EQ. 262) THEN
         IVAL = READ_TIFF_SHORT(ENT(9))
         PHOTOINTERP = IVAL
         IF (DEBUG) WRITE(*,*) 'PhotometricInterpretation = ', IVAL
      ELSE IF (TAG .EQ. 273) THEN
         IF (COUNT .EQ. 1) THEN
            IF (TYPE .EQ. 3) IVAL = READ_TIFF_SHORT(ENT(9))
            IF (TYPE .EQ. 4) IVAL = READ_TIFF_LONG(ENT(9))
            STRIPOFFSETS(1) = IVAL
            IF (DEBUG) WRITE(*,*) 'StripOffsets(1) = ', IVAL
         ELSE IF (COUNT .LT. MAXSTRIPS) THEN
            NUMSTRIPS = COUNT
            OFFSET    = READ_TIFF_LONG(ENT(9))
            DO I = 1, NUMSTRIPS
               IF (TYPE .EQ. 3) THEN
                  IF (READ_TIFF_FILE(BUF2,
     $                               OFFSET+(I-1)*2,
     $                               OFFSET+I*2-1) .NE. 0) GOTO 931
                  IVAL = READ_TIFF_SHORT(BUF2)
               ENDIF
               IF (TYPE .EQ. 4) THEN
                  IF (READ_TIFF_FILE(BUF4,
     $                               OFFSET+(I-1)*4,
     $                               OFFSET+I*4-1) .NE. 0) GOTO 931
                  IVAL = READ_TIFF_LONG(BUF4)
               ENDIF
               STRIPOFFSETS(I) = IVAL
               IF (DEBUG) WRITE(*,*) 'StripOffsets(',I,') = ', IVAL
            ENDDO
         ELSE
            WRITE(*,*) '** PARSE_IFD_ENT: Too many data strips in file'
            WRITE(*,*) '** NUMSTRIPS = ', COUNT
            WRITE(*,*) '** Cannot handle more than ',MAXSTRIPS,' strips'
            WRITE(*,*) '** Increase MAXSTRIPS and recompile.'
            WRITE(*,*) '** Quitting.'
            STOP
         ENDIF
      ELSE IF (TAG .EQ. 277) THEN
         IVAL = READ_TIFF_SHORT(ENT(9))
         SAMPLESPERPIXEL = IVAL
         IF (DEBUG) WRITE(*,*) 'SamplesPerPixel = ', IVAL
      ELSE IF (TAG .EQ. 278) THEN
         IF (TYPE .EQ. 3) IVAL = READ_TIFF_SHORT(ENT(9))
         IF (TYPE .EQ. 4) IVAL = READ_TIFF_LONG(ENT(9))
         ROWSPERSTRIP = IVAL
         IF (DEBUG) WRITE(*,*) 'RowsPerStrip = ', IVAL
      ELSE IF (TAG .EQ. 279) THEN
         IF (COUNT .EQ. NUMSTRIPS) THEN
            IF (COUNT .EQ. 1) THEN
               IF (TYPE .EQ. 3) IVAL = READ_TIFF_SHORT(ENT(9))
               IF (TYPE .EQ. 4) IVAL = READ_TIFF_LONG(ENT(9))
               STRIPBYTECOUNTS(1) = IVAL
               IF (DEBUG) WRITE(*,*) 'StripByteCounts(1) = ', IVAL
            ELSE
               OFFSET = READ_TIFF_LONG(ENT(9))
               DO I = 1, NUMSTRIPS
                  IF (TYPE .EQ. 3) THEN
                     IF (READ_TIFF_FILE(BUF2,
     $                                  OFFSET+(I-1)*2,
     $                                  OFFSET+I*2-1) .NE. 0) GOTO 931
                     IVAL = READ_TIFF_SHORT(BUF2)
                  ENDIF
                  IF (TYPE .EQ. 4) THEN
                     IF (READ_TIFF_FILE(BUF4,
     $                                  OFFSET+(I-1)*4,
     $                                  OFFSET+I*4-1) .NE. 0) GOTO 931
                     IVAL = READ_TIFF_LONG(BUF4)
                  ENDIF
                  STRIPBYTECOUNTS(I) = IVAL
                  IF (DEBUG)
     $                 WRITE(*,*) 'StripByteCounts(',I,') = ', IVAL
               ENDDO
            ENDIF
         ELSE
            WRITE(*,*) '** PARSE_IFD_ENT: Badly formed TIFF file.'
            WRITE(*,*) '** Number of StripByteCounts not the same'
            WRITE(*,*) '** as the number of StripOffsets.'
            WRITE(*,*) '** Quitting.'
            STOP
         ENDIF
      ELSE IF (TAG .EQ. 305) THEN
         IF (COUNT .LE. 4) THEN
            DO I = 1, COUNT
               SOFTWARE(I) = CHAR(ENT(9+I))
            ENDDO
         ELSE
            OFFSET = READ_TIFF_LONG(ENT(9))
            DO I = 1, COUNT
               IF (READ_TIFF_FILE(BUF1,
     $                            OFFSET+(I-1),
     $                            OFFSET+I-1) .NE. 0) GOTO 931
               SOFTWARE(I) = CHAR(BUF1)
            ENDDO
         ENDIF
         IF (DEBUG) WRITE(*,*) 'Software = ',
     $        (SOFTWARE(I), I = 1, COUNT-1)
      ELSE IF (TAG .EQ. 306) THEN
         IF (COUNT .LE. 4) THEN
            DO I = 1, COUNT
               DATETIME(I) = CHAR(ENT(9+I))
            ENDDO
         ELSE
            OFFSET = READ_TIFF_LONG(ENT(9))
            DO I = 1, COUNT
               IF (READ_TIFF_FILE(BUF1,
     $                            OFFSET+(I-1),
     $                            OFFSET+I-1) .NE. 0) GOTO 931
               DATETIME(I) = CHAR(BUF1)
            ENDDO
         ENDIF
         IF (DEBUG) WRITE(*,*) 'DateTime = ',
     $        (DATETIME(I), I = 1, COUNT-1)
      ELSE IF (DEBUG) THEN
         WRITE(*,*) 'TAG = ',TAG,' TYPE = ',TYPE,' COUNT = ',COUNT
      ENDIF
C
      PARSE_IFD_ENT = 0
      RETURN
C
C     Error trap
C
 931  PARSE_IFD_ENT = 1
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_DATA
C
C     Description:
C        Reads the image data into the DATA.
C        NB. DATA has to be big enough to contain all the image
C        data: i.e DATA(WIDTH*HEIGHT)
C
C        Returns 0 if successful, 1 otherwise.
C
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_DATA(DATA, DEBUG)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C
      INTEGER DATA(*)           ! Array to contain the image data
      LOGICAL DEBUG             ! Debug mode
C
C     Functions
C
      INTEGER READ_TIFF_FILE
      INTEGER READ_TIFF_SHORT
      INTEGER READ_TIFF_LONG
C
C     Variables
C
      INTEGER I, J              ! Loop counterS
      INTEGER PIXREAD           ! Num pixels read so far
C
      PIXREAD = 0
      DO I = 1, NUMSTRIPS
         IF (STRIPBYTECOUNTS(I) .GT. MAXSTRPSZ) THEN
            WRITE(*,*) '** READ_TIFF_DATA: Strip size too large'
            WRITE(*,*) '** SIZE = ', STRIPBYTECOUNTS(I)
            WRITE(*,*) '** MAXSTRPSZ = ', MAXSTRPSZ
            WRITE(*,*) '** Increase MAXSTRPSZ and recompile'
            WRITE(*,*) '** Quitting'
            STOP
         ENDIF
C
         IF (READ_TIFF_FILE(RAWDATA,
     $        STRIPOFFSETS(I), 
     $        STRIPOFFSETS(I)+STRIPBYTECOUNTS(I)-1) .NE. 0) 
     $        GOTO 941
C
         IF (BITSPERSAMPLE .EQ. 8) THEN
            DO J = 1, STRIPBYTECOUNTS(I)
               DATA(PIXREAD+1) = RAWDATA(J)
               PIXREAD = PIXREAD + 1
            ENDDO
         ELSE IF (BITSPERSAMPLE .EQ. 16) THEN
            DO J = 1, STRIPBYTECOUNTS(I), 2
               DATA(PIXREAD+1) = READ_TIFF_SHORT(RAWDATA(J))
               PIXREAD = PIXREAD + 1
            ENDDO
         ELSE IF (BITSPERSAMPLE .EQ. 32) THEN
            DO J = 1, STRIPBYTECOUNTS(I), 4
               DATA(PIXREAD+1) = READ_TIFF_LONG(RAWDATA(J))
               PIXREAD = PIXREAD + 1
            ENDDO
         ENDIF
         IF (DEBUG) WRITE(*,*) 'STRIP = ',I,' PIXREAD = ', PIXREAD
      ENDDO
C
      READ_TIFF_DATA = 0
      RETURN
C     
C     Error trap
C     
 941  READ_TIFF_DATA = 1
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: TIFF_IMAGE_WIDTH
C
C     Description:
C        Returns the width of the currently open TIFF image
C
C**********************************************************************
      INTEGER FUNCTION TIFF_IMAGE_WIDTH()
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
      TIFF_IMAGE_WIDTH = WIDTH
      END
C
C
C
C**********************************************************************
C     Routine: TIFF_IMAGE_HEIGHT
C
C     Description:
C        Returns the height of the currently open TIFF image
C
C**********************************************************************
      INTEGER FUNCTION TIFF_IMAGE_HEIGHT()
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
      TIFF_IMAGE_HEIGHT = HEIGHT
      END
C
C
C
C**********************************************************************
C     Routine: CLOSE_TIFF_FILE
C
C     Description:
C        Closes the TIFF file.
C**********************************************************************
      SUBROUTINE CLOSE_TIFF_FILE()
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
      CLOSE(UNIT=UNT)
      END
C
C     
C
C**********************************************************************
C     Routine: BIGEND_CPU
C
C     Description:
C        Returns .TRUE. if the machine is big endian
C**********************************************************************
      LOGICAL FUNCTION BIGEND_CPU()
      IMPLICIT NONE
C
C     Variables
C          
      BYTE         B(2)
      INTEGER*2    I2
      EQUIVALENCE (B, I2)
C
      I2 = 1
      BIGEND_CPU = (B(1) .NE. 1)
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_SHORT
C
C     Description:
C        Converts the 2 bytes in RAW to INTEGER*2 swapping
C        the bytes if necessary.
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_SHORT(RAW)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C     
      BYTE         RAW(2)
C
C     Variables
C
      BYTE         B(2)
      INTEGER*2    I2
      EQUIVALENCE (B, I2)
C
      IF (SWAP) THEN
         B(1) = RAW(2)
         B(2) = RAW(1)
      ELSE
         B(1) = RAW(1)
         B(2) = RAW(2)
      ENDIF
C
C     The integer is unsigned so convert if necessary.
C
      IF (I2 .LT. 0) THEN
        READ_TIFF_SHORT = 2**16 + I2
      ELSE
         READ_TIFF_SHORT = I2
      ENDIF
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_TIFF_LONG
C
C     Description:
C        Converts the 4 bytes in RAW to INTEGER*4 swapping
C        the bytes if necessary.
C**********************************************************************
      INTEGER FUNCTION READ_TIFF_LONG(RAW)
      IMPLICIT NONE
      INCLUDE 'tiff.inc'
C
C     Parameters
C     
      BYTE         RAW(4)
C
C     Variables
C          
      BYTE         B(4)
      INTEGER*4    I4
      EQUIVALENCE (B, I4)
C
      IF (SWAP) THEN
         B(1) = RAW(4)
         B(2) = RAW(3)
         B(3) = RAW(2)
         B(4) = RAW(1)
      ELSE
         B(1) = RAW(1)
         B(2) = RAW(2)
         B(3) = RAW(3)
         B(4) = RAW(4)
      ENDIF
C
      READ_TIFF_LONG = I4
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: STRLEN
C
C     Description:
C        Returns the number of characters in STRING not including
C        non-printable characters.
C**********************************************************************
      INTEGER FUNCTION STRLEN(STRING)
      IMPLICIT NONE
      CHARACTER*(*) STRING
C
C     Returns the length of a string minus whitespace at the end.
C
      INTEGER I
C
      DO I = LEN(STRING), 1, -1
         IF (STRING(I:I) .GT. ' ') THEN
            GOTO 20
         ENDIF
      ENDDO
C
 20   STRLEN = I
      RETURN
      END
C
